﻿Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.Linq
Imports System.Xml.Linq
Imports System.Windows.Media.Imaging
Imports System.IO
Imports WpfControlLibrary.OrgCharts
Imports Infragistics
Imports Infragistics.Controls.Maps
Imports Infragistics.DragDrop
Imports Infragistics.Controls.Menus
Imports Infragistics.Windows.Controls

Public Class OrgChart

    Private OrganizationModel As New OrganizationViewModel()

    Private currentDragged As OrgChartNodeControl

    Private nodeDragTemplate As DataTemplate

    Public Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        nodeDragTemplate = New DataTemplate(GetType(Employee))

    End Sub

    Public Function CreateChart(ByVal sXML As String) As Boolean

        Me.OrganizationModel.Employees = New ObservableCollection(Of Employee)()

        Try

            Dim xd As XDocument
            xd = XDocument.Parse(sXML)

            For Each xe As XElement In xd.Nodes
                If xe.Name = "EmployeeData" Then
                    For Each xf As XElement In xe.Nodes
                        If xf.Name = "Employees" Then
                            Dim NewEmployee As New Employee
                            CreateNodes(xf.Nodes, NewEmployee)
                            Me.OrganizationModel.Employees.Add(NewEmployee)
                        End If
                    Next
                End If
            Next

        Catch ex As Exception

            Debug.Print(ex.Message)

            Return False

        End Try

        Me.OrgChart.DataContext = Me.OrganizationModel

        Return True

    End Function

    Public Sub SetScaleToFit()

        Me.OrgChart.ScaleToFit()

    End Sub

    Private Sub CreateNodes(ByVal nodes As Object, ByRef ThisEmployee As Employee)

        ThisEmployee.Subordinates = New ObservableCollection(Of Employee)

        For Each xe As XElement In nodes

            If xe.Name = "EmpID" Then
                ThisEmployee.Id = xe.Value
            End If
            If xe.Name = "EmpName" Then
                ThisEmployee.Name = xe.Value
            End If
            If xe.Name = "EmpAttr" Then
                ThisEmployee.Attributes = xe.Value
            End If

            If xe.Name = "Employees" Then
                Dim NewEmployee As New Employee
                CreateNodes(xe.Nodes, NewEmployee)
                ThisEmployee.Subordinates.Add(NewEmployee)
            End If

        Next

    End Sub

    Private Sub OrgChart_NodeControlAttachedEvent(sender As Object, e As OrgChartNodeEventArgs)

        Dim employee As Employee = DirectCast(e.Node.Node.Data, Employee)
        If employee.Attributes <> "" Then e.Node.Background = New SolidColorBrush(ColorConverter.FromString(employee.Attributes))

        Dim dataType As String = e.Node.Node.Data.[GetType]().Name

        'Create a new DragSource object.
        Dim dragSource As New DragSource()
        dragSource.IsDraggable = True    'dragSource.DragChannels = assign drag channels
        dragSource.DragTemplate = nodeDragTemplate
        AddHandler dragSource.DragStart, AddressOf Node_DragStart
        AddHandler dragSource.Drop, AddressOf Node_Drop

        'Make the Node a Drag Source.
        DragDropManager.SetDragSource(e.Node, dragSource)

        'Create a new DropTarget object.
        Dim dropTarget As New DropTarget()
        dropTarget.IsDropTarget = True    'dropTarget.DropChannels = assign drop channels

        'Make the Node a Drop Target.
        DragDropManager.SetDropTarget(e.Node, dropTarget)

    End Sub

    Private Sub OrgChart_NodeControlDetachedEvent(sender As Object, e As OrgChartNodeEventArgs)

        If DragDropManager.IsDragging AndAlso e.Node Is currentDragged Then
            DragDropManager.EndDrag(True)
        End If

    End Sub

    Private Sub Node_DragStart(sender As Object, e As DragDropStartEventArgs)

        'Assing the Employee of the dragged node as data context for the Dragged Item.
        'This is needed for the DragTemplate.
        e.Data = DirectCast(e.DragSource, OrgChartNodeControl).Node.Data
        currentDragged = DirectCast(e.DragSource, OrgChartNodeControl)

    End Sub

    Private Sub Node_Drop(sender As Object, e As DropEventArgs)

        ''Check for a recursive relation between the drag source and the drop target.
        'Dim levels As Integer = DragSource.OrgChart.ActualDepth - DragSource.Level
        'If DragSource.ContainsNode(levels, DropTarget.Data) Then
        '    Return
        'End If

        ''Remove the dragged node from its parent.
        'DragSource.GetSiblingsCollection(collectionName).InvokeMethod("Remove", New Object() {DragSource.Data})
        ''Add the dragged node to the drop target node.
        'DropTarget.GetChildrenCollection(collectionName).InvokeMethod("Add", New Object() {DragSource.Data})

        Dim DropOnNode As Boolean = False

        'Get the dragged OrgChartNodeControl object.
        Dim draggedNodeControl As OrgChartNodeControl = TryCast(e.DragSource, OrgChartNodeControl)
        'Get the dragged OrgChartNode object.
        Dim draggedNode As OrgChartNode = draggedNodeControl.Node
        'Get the underlying data.
        Dim draggedData As Object = draggedNode.Data
        Dim draggedEmployee As Employee = Nothing
        FindEmployee(draggedData.Id, Me.OrganizationModel.Employees, draggedEmployee)

        If IsNothing(draggedEmployee) Then Exit Sub

        'Get the dropped OrgChartNodeControl object.
        Dim droppedEmployee As Employee = Nothing
        Dim droppedNodeControl As OrgChartNodeControl = TryCast(e.DropTarget, OrgChartNodeControl)
        If Not IsNothing(droppedNodeControl) Then
            DropOnNode = True
            'Get the dropped OrgChartNode object.
            Dim droppedNode As OrgChartNode = droppedNodeControl.Node
            'Get the underlying data.
            Dim droppedData As Object = droppedNode.Data
            FindEmployee(droppedData.Id, Me.OrganizationModel.Employees, droppedEmployee)
        End If

        If IsNothing(droppedEmployee) Then Exit Sub

        'Prevent drop on self.
        If draggedEmployee.Id = droppedEmployee.Id Then Exit Sub

        'Check if the drop target already contains the dragged node.
        If droppedEmployee.Subordinates.Contains(draggedEmployee) Then Exit Sub

        'Remove the dragged Employee from old node
        If Me.OrganizationModel.Employees.Contains(draggedEmployee) Then
            Me.OrganizationModel.Employees.Remove(draggedEmployee)
        Else
            RemoveEmployee(draggedEmployee, Me.OrganizationModel.Employees)
        End If

        'Add the dragged employee to new node
        If DropOnNode Then
            If Not IsNothing(droppedEmployee) Then
                droppedEmployee.Subordinates.Add(draggedEmployee)
            End If
        Else
            Me.OrganizationModel.Employees.Add(draggedEmployee)
        End If

    End Sub

    Private Sub FindEmployee(ByVal Id As String, ByRef Employees As ObservableCollection(Of Employee), ByRef FoundEmployee As Employee)

        If IsNothing(FoundEmployee) Then
            For Each emp As Employee In Employees
                If IsNothing(FoundEmployee) Then
                    If emp.Id = Id Then
                        FoundEmployee = emp
                    Else
                        FindEmployee(Id, emp.Subordinates, FoundEmployee)
                    End If
                End If
            Next
        End If

    End Sub

    Private Sub RemoveEmployee(ByVal Employee As Employee, ByRef Employees As ObservableCollection(Of Employee))

        If Not IsNothing(Employee) Then
            For Each emp As Employee In Employees
                If emp.Subordinates.Count > 0 Then
                    If emp.Subordinates.Contains(Employee) Then
                        emp.Subordinates.Remove(Employee)
                    Else
                        RemoveEmployee(Employee, emp.Subordinates)
                    End If
                End If
            Next
        End If

    End Sub

    Private Sub XamContextMenu_ItemClicked(sender As Object, e As ItemClickedEventArgs)

        Select Case e.Item.Name
            Case "Zoom100", "ZoomScale"
                SetZoom(e.Item.Name)
            Case "DirAbove", "DirBelow", "DirLeft", "DirRight"
                SetNodeDirection(e.Item.Name)
            Case "ArrOrthogonal", "ArrStagger"
                SetNodeArrangement(e.Item.Name)
            Case "ExpVisible", "ExpHidden"
                SetExpansionVisibilty(e.Item.Name)
            Case "OvpVisible", "OvpHidden"
                SetOverviewVisibilty(e.Item.Name)
            Case "Lvl0", "Lvl1", "Lvl2", "Lvl3", "Lvl4", "Lvl5", "Lvl6"
                SetNodeLevel(e.Item.Name.Substring(3, 1))
            Case "ExpPNG", "ExpJPEG"
                SetExport(e.Item.Name)
            Case "AddPositions"
                AddNewPositions()
        End Select

    End Sub

    Private Sub XamMenuItem_Click(sender As Object, e As EventArgs)
        'MessageBox.Show(MenuStrings.CM_MenuItem1Clicked)
    End Sub

    Private Sub XamContextMenu_Opening(sender As Object, e As OpeningEventArgs)
        'Dim elements = e.GetClickedElements(Of BlueRectangle)()
        'If elements.Count <> 0 Then
        '    elements(0).TextForeground = New SolidColorBrush(Colors.LightGray)
        'End If
    End Sub

    Private Sub XamContextMenu_Closing(sender As Object, e As Infragistics.Controls.Menus.ItemClickedEventArgs)
        'rectangle.TextForeground = New SolidColorBrush(Colors.White)
    End Sub

    Private Sub SetZoom(ByVal eType As String)

        Select Case eType
            Case "Zoom100"
                OrgChart.ZoomTo100()
            Case "ZoomScale"
                OrgChart.ScaleToFit()
        End Select

    End Sub

    Private Sub SetNodeDirection(ByVal eType As String)

        Select Case eType
            Case "DirAbove"
                OrgChart.NodeFlowDirection = NodeFlowDirection.Above
            Case "DirBelow"
                OrgChart.NodeFlowDirection = NodeFlowDirection.Below
            Case "DirLeft"
                OrgChart.NodeFlowDirection = NodeFlowDirection.Left
            Case "DirRight"
                OrgChart.NodeFlowDirection = NodeFlowDirection.Right
        End Select

    End Sub

    Private Sub SetNodeArrangement(ByVal eType As String)

        Select Case eType
            Case "ArrOrthogonal"
                OrgChart.NodeArrangementStrategy = NodeArrangementStrategy.Orthogonal
            Case "ArrStagger"
                OrgChart.NodeArrangementStrategy = NodeArrangementStrategy.Stagger
        End Select

    End Sub

    Private Sub SetExpansionVisibilty(ByVal eType As String)

        Select Case eType
            Case "ExpVisible"
                OrgChart.ExpansionIndicatorVisibility = System.Windows.Visibility.Visible
            Case "ExpHidden"
                OrgChart.ExpansionIndicatorVisibility = System.Windows.Visibility.Collapsed
        End Select

    End Sub

    Private Sub SetOverviewVisibilty(ByVal eType As String)

        Select Case eType
            Case "OvpVisible"
                OrgChart.OverviewPlusDetailPaneVisibility = System.Windows.Visibility.Visible
            Case "OvpHidden"
                OrgChart.OverviewPlusDetailPaneVisibility = System.Windows.Visibility.Collapsed
        End Select

    End Sub

    Private Sub SetNodeLevel(ByVal eType As String)

        Dim i As Integer

        Try
            i = CInt(eType)
        Catch ex As Exception
            Exit Sub
        End Try

        If i = 0 Then i = -1

        OrgChart.MaxDepth = i

    End Sub

    Private Sub SetExport(ByVal eType As String)

        Dim vis As Boolean = False

        If OrgChart.OverviewPlusDetailPaneVisibility = System.Windows.Visibility.Visible Then
            vis = True
            OrgChart.OverviewPlusDetailPaneVisibility = System.Windows.Visibility.Collapsed
        End If

        Dim width As Integer = CInt(OrgChart.ActualWidth)
        Dim height As Integer = CInt(OrgChart.ActualHeight)

        Dim bmp As New RenderTargetBitmap(width, height, 96, 96, PixelFormats.Pbgra32)
        bmp.Render(OrgChart)

        Dim fln As String = ""

        Select Case eType
            Case "ExpJPEG"
                Dim jpg As New JpegBitmapEncoder()
                jpg.Frames.Add(BitmapFrame.Create(bmp))
                fln = "C:\TestChart.jpg"
                Using stm As Stream = File.Create(fln)
                    jpg.Save(stm)
                End Using
            Case "ExpPNG"
                Dim png As New PngBitmapEncoder()
                png.Frames.Add(BitmapFrame.Create(bmp))
                fln = "C:\TestChart.png"
                Using stm As Stream = File.Create(fln)
                    png.Save(stm)
                End Using
        End Select

        Process.Start(fln)

        If vis Then
            OrgChart.OverviewPlusDetailPaneVisibility = System.Windows.Visibility.Visible
        End If

    End Sub

    Private Sub AddNewPositions()

        System.Diagnostics.Debug.WriteLine(OrgChart.MaxDepth.ToString)

    End Sub

    Private Sub OrgChart_NodeMouseLeftButtonDown(sender As Object, e As Infragistics.Controls.Maps.OrgChartNodeClickEventArgs) Handles OrgChart.NodeMouseLeftButtonDown

        If e.MouseEventArgs.ClickCount = 2 Then
            ' mouse double click
            Dim employee As Employee = DirectCast(e.Node.Node.Data, Employee)
            employee.Name = "Jo Smith"
        End If

    End Sub

End Class
